package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yitter.idgen.YitIdHelper;
import com.xy.collections.list.JArrayList;
import com.xy.collections.list.JList;
import com.xy.collections.map.JMap;
import com.xy.device.EnumDeviceCharging;
import com.xy.device.EnumSimConfig;
import com.xy.dto.DeviceChargingDto;
import com.xy.dto.DeviceSimDto;
import com.xy.entity.DeviceSim;
import com.xy.entity.DeviceSimCharge;
import com.xy.mapper.DeviceSimMapper;
import com.xy.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;


/**
 * <p>
 * 设备流量卡 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-10-16
 */
@Service
@AllArgsConstructor
@Api(tags = "设备流量卡")
public class DeviceSimServiceImpl extends ServiceImpl<DeviceSimMapper, DeviceSim> implements DeviceSimService {

    private DeviceSimChargeServiceImpl deviceSimChargeService;

    @Override
    @ApiOperation("购买")
    public R pay(DeviceSimDto.Pay pay) {
        LocalDateTime now = LocalDateTime.now();
        List<DeviceSimDto.Pay.DeviceSimCharge> deviceSimCharges = pay.getDeviceSimCharges();
        List<DeviceSimCharge> deviceSimChargess = new ArrayList<>(deviceSimCharges.size());
        deviceSimCharges.forEach(deviceSimCharge -> {
            DeviceSimCharge deviceSimChargeInfo = new DeviceSimCharge()
                    .setId(YitIdHelper.nextId())
                    .setOrderId(pay.getOrderId())
                    .setSimId(deviceSimCharge.getSimId())
                    .setMercId(pay.getMercId())
                    .setMoney(deviceSimCharge.getMoney())
                    .setSize(deviceSimCharge.getSize())
                    .setPayType(pay.getPayType())
                    .setNote(pay.getNote())
                    .setCreateTime(now)
                    .setUpdateTime(now);
            deviceSimChargess.add(deviceSimChargeInfo);
        });
        deviceSimChargeService.saveBatch(deviceSimChargess);
        return R.ok();
    }

    @Override
    @ApiOperation("购买回调")
    public R payNotice(DeviceSimDto.PayNotice payNotice) {
        //查询设备流量卡充值表
        List<DeviceSimCharge> list = deviceSimChargeService.list(new LambdaQueryWrapper<DeviceSimCharge>().eq(DeviceSimCharge::getOrderId, payNotice.getOrderId()));
        if (!Emptys.check(list)) {
            return R.ok();
        }
        //查询设备流量卡
        JList<DeviceSimCharge> deviceSimCharges = new JArrayList<>(list);
        JMap<String, DeviceSimCharge> deviceSimChargesJMaps = deviceSimCharges.toMap(DeviceSimCharge::getSimId).cover();
        List<DeviceSim> deviceSims = listByIds(deviceSimCharges.getProperty(DeviceSimCharge::getSimId));
        //循环处理
        LocalDateTime now = LocalDateTime.now();
        deviceSims.forEach(deviceSim -> {
            DeviceSimCharge deviceSimCharge = deviceSimChargesJMaps.get(deviceSim.getId());
            String newTimeout = deviceSimCharge.getPayType() == 100 ? DataTime.getStringAround(0, 0, deviceSimCharge.getSize(), 0, 0, 0)
                    : DataTime.getStringAround(deviceSimCharge.getSize(), 0, 0, 0, 0, 0);
            deviceSim.setTimeout(DataTime.toLocal(newTimeout))
                    .setLastRenewalTime(now)
                    .setUpdateTime(now);
        });
        updateBatchById(deviceSims);
        return R.ok();
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<DeviceSimDto.PageVo>> page(@RequestBody DeviceSimDto.Page page) {
        Integer value = SysDictUtils.getValue(EnumSimConfig.Code.CODE.getCode(), EnumSimConfig.N_200.getCode(), Integer.class);
        String theTime = DataTime.getStringAround(0, 0, value, 0, 0, 0);
        page.setThisTime(LocalDateTime.now()).setTheTime(DataTime.toLocal(theTime));
        IPage<DeviceSimDto.PageVo> iPage = baseMapper.page(toIPage(page.getPage()), page);
        List<DeviceSimDto.PageVo> records = iPage.getRecords();
        if (Emptys.check(records)) {
            //封装过期状态说明
            records.forEach(record -> {
                String timeoutStatus;
                DeviceSimDto.Vo sim = record.getSim();
                LocalDateTime timeout = sim.getTimeout();
                long s = DataTime.diff(page.getThisTime(), timeout, "s");
                if (s <= 0) {
                    timeoutStatus = "欠费(" + (~(s / 86400 - 1)) + "天)";
                } else {
                    timeoutStatus = s <= value * 86400 ? "即将到期(" + s / 86400 + "天)" : "正常(" + s / 86400 + "天)";
                }
                record.getSim().setTimeoutStatus(timeoutStatus);
            });
        }
        return R.ok(toPageBean(iPage));
    }
}